use strict;
use Test::More tests => 3;

BEGIN { use_ok('Catalyst::Model::DBIC::Schema') }
BEGIN { use_ok('Catalyst::Helper::Model::DBIC::Schema') }
BEGIN { use_ok('Catalyst::Helper::Model::DBIC::SchemaLoader') }
